@@ -67,6 +67,7 @@ generally do their best to write you a useful test.  You can also use
     public domain dedication, so you can use it without any restrictions.
 """
 
+import ast
 import builtins
 import contextlib
 import enum
@@ -83,6 +84,7 @@ from typing import (
     Any,
     Callable,
     Dict,
+    List,
     Mapping,
     Optional,
     Set,
@@ -195,14 +197,98 @@ def _exceptions_from_docstring(doc: str) -> Tuple[Type[Exception], ...]:
 # on analysis of type-annotated code to detect arguments which almost always
 # take values of a particular type.
 _GUESS_STRATEGIES_BY_NAME = (
+    (st.integers(0, 32), ["ndims"]),
+    (st.booleans(), ["keepdims"]),
     (st.text(), ["name", "filename", "fname"]),
     (st.floats(), ["real", "imag"]),
     (st.functions(), ["function", "func", "f"]),
+    (st.functions(returns=st.booleans(), pure=True), ["pred", "predicate"]),
     (st.iterables(st.integers()) | st.iterables(st.text()), ["iterable"]),
 )
 
 
-def _strategy_for(param: inspect.Parameter) -> Union[st.SearchStrategy, InferType]:
+def _type_from_doc_fragment(token: str) -> Optional[type]:
+    # Special cases for "integer" and for numpy array-like and dtype
+    if token == "integer":
+        return int
+    if "numpy" in sys.modules:
+        if re.fullmatch(r"[Aa]rray[-_ ]?like", token):
+            return sys.modules["numpy"].ndarray  # type: ignore
+        elif token == "dtype":
+            return sys.modules["numpy"].dtype  # type: ignore
+    # Natural-language syntax, e.g. "sequence of integers"
+    coll_match = re.fullmatch(r"(\w+) of (\w+)", token)
+    if coll_match is not None:
+        coll_token, elem_token = coll_match.groups()
+        elems = _type_from_doc_fragment(elem_token)
+        if elems is None and elem_token.endswith("s"):
+            elems = _type_from_doc_fragment(elem_token[:-1])
+        if elems is not None and coll_token in ("list", "sequence", "collection"):
+            return List[elems]  # type: ignore
+        # This might be e.g. "array-like of float"; arrays is better than nothing
+        # even if we can't conveniently pass a generic type around.
+        return _type_from_doc_fragment(coll_token)
+    # Check either builtins, or the module for a dotted name
+    if "." not in token:
+        return getattr(builtins, token, None)
+    mod, name = token.rsplit(".", maxsplit=1)
+    return getattr(sys.modules.get(mod, None), name, None)
+
+
+def _strategy_for(
+    param: inspect.Parameter,
+    docstring: str,
+) -> Union[st.SearchStrategy, InferType]:
+    # Example types in docstrings:
+    # - `:type a: sequence of integers`
+    # - `b (list, tuple, or None): ...`
+    # - `c : {"foo", "bar", or None}`
+    for pattern in (
+        fr"^\s*\:type\s+{param.name}\:\s+(.+)",  # RST-style
+        fr"^\s*{param.name} \((.+)\):",  # Google-style
+        fr"^\s*{param.name} \: (.+)",  # Numpy-style
+    ):
+        match = re.search(pattern, docstring, flags=re.MULTILINE)
+        if match is None:
+            continue
+        doc_type = match.group(1)
+        if doc_type.endswith(", optional"):
+            # Convention to describe "argument may be omitted"
+            doc_type = doc_type[: -len(", optional")]
+        doc_type = doc_type.strip("}{")
+        elements = []
+        types = []
+        for token in re.split(r",? +or +| *, *", doc_type):
+            for prefix in ("default ", "python "):
+                # `str or None, default "auto"`; `python int or numpy.int64`
+                if token.startswith(prefix):
+                    token = token[len(prefix) :]
+            if not token:
+                continue
+            try:
+                # Elements of `{"inner", "outer"}` etc.
+                elements.append(ast.literal_eval(token))
+                continue
+            except (ValueError, SyntaxError):
+                t = _type_from_doc_fragment(token)
+                if isinstance(t, type) or is_generic_type(t):
+                    assert t is not None
+                    types.append(t)
+        if (
+            param.default is not inspect.Parameter.empty
+            and param.default not in elements
+            and not isinstance(
+                param.default, tuple(t for t in types if isinstance(t, type))
+            )
+        ):
+            with contextlib.suppress(SyntaxError):
+                compile(repr(st.just(param.default)), "<string>", "eval")
+                elements.insert(0, param.default)
+        if elements or types:
+            return (st.sampled_from(elements) if elements else st.nothing()) | (
+                st.one_of(*map(st.from_type, types)) if types else st.nothing()
+            )
+
     # If our default value is an Enum or a boolean, we assume that any value
     # of that type is acceptable.  Otherwise, we only generate the default.
     if isinstance(param.default, bool):
@@ -306,8 +392,10 @@ def _get_strategies(
         if pass_result_to_next_func and i >= 1:
             del params[next(iter(params))]
         hints = get_type_hints(f)
+        docstring = getattr(f, "__doc__", None) or ""
         builder_args = {
-            k: infer if k in hints else _strategy_for(v) for k, v in params.items()
+            k: infer if k in hints else _strategy_for(v, docstring)
+            for k, v in params.items()
         }
         with _with_any_registered():
             strat = st.builds(f, **builder_args).wrapped_strategy  # type: ignore
@@ -436,7 +524,7 @@ def _valid_syntax_repr(strategy):
         # .filter(), and .flatmap() to work without NameError
         imports = {i for i in _imports_for_strategy(strategy) if i[1] in r}
         return imports, r
-    except (SyntaxError, InvalidArgument):
+    except (SyntaxError, RecursionError, InvalidArgument):
         return set(), "nothing()"
 
 
@@ -601,7 +689,7 @@ def _make_test(imports: Set[Union[str, Tuple[str, str]]], body: str) -> str:
     from_ = {
         "from {} import {}".format(module, ", ".join(sorted(names)))
         for module, names in from_imports.items()
-        if module not in do_not_import
+        if isinstance(module, str) and module not in do_not_import
     }
     header = IMPORT_SECTION.format(imports="\n".join(sorted(direct) + sorted(from_)))
     nothings = body.count("st.nothing()")
@@ -682,9 +770,12 @@ def magic(
                     v
                     for k, v in vars(thing).items()
                     if callable(v)
-                    and (getattr(v, "__module__", pkg) == pkg or not pkg)
+                    and not is_mock(v)
+                    and ((not pkg) or getattr(v, "__module__", pkg).startswith(pkg))
                     and not k.startswith("_")
                 ]
+                if pkg and any(getattr(f, "__module__", pkg) == pkg for f in funcs):
+                    funcs = [f for f in funcs if getattr(f, "__module__", pkg) == pkg]
             for f in funcs:
                 try:
                     if (
@@ -1229,6 +1320,12 @@ def _make_ufunc_body(func, *, except_, style):
     )
 
     qname = _get_qualname(func, include_module=True)
+    obj_sigs = ["O" in sig for sig in func.types]
+    if all(obj_sigs) or not any(obj_sigs):
+        types = f"sampled_from({qname}.types)"
+    else:
+        types = f"sampled_from([sig for sig in {qname}.types if 'O' not in sig])"
+
     return _make_test_body(
         func,
         test_body=dedent(body).strip(),
@@ -1236,9 +1333,5 @@ def _make_ufunc_body(func, *, except_, style):
         assertions=assertions,
         ghost="ufunc" if func.signature is None else "gufunc",
         style=style,
-        given_strategies={
-            "data": st.data(),
-            "shapes": shapes,
-            "types": f"sampled_from([sig for sig in {qname}.types if 'O' not in sig])",
-        },
+        given_strategies={"data": st.data(), "shapes": shapes, "types": types},
     )
